@@ -60,6 +60,8 @@
 
 #include "vsearch.h"
 
+//#define SHOW_RUSAGE
+
 static const char * progress_prompt;
 static uint64_t progress_next;
 static uint64_t progress_size;
@@ -112,7 +114,7 @@ void  __attribute__((noreturn)) fatal(const char * msg)
   fprintf(stderr, "\n\n");
   fprintf(stderr, "Fatal error: %s\n", msg);
 
-  if (opt_log)
+  if (fp_log)
     {
       fprintf(fp_log, "\n\n");
       fprintf(fp_log, "Fatal error: %s\n", msg);
@@ -186,7 +188,7 @@ int64_t getusec(void)
 
 void show_rusage()
 {
-#if 0
+#ifdef SHOW_RUSAGE
   double user_time = 0.0;
   double system_time = 0.0;
 
