@@ -58,7 +58,9 @@
 
 */
 
+#define __STDC_CONSTANT_MACROS 1
 #define __STDC_FORMAT_MACROS 1
+#define __STDC_LIMIT_MACROS 1
 #define __restrict
 
 #ifdef HAVE_CONFIG_H
@@ -87,13 +89,24 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <float.h>
+#include <stdint.h>
 #include <inttypes.h>
 #include <stdarg.h>
 
 #define PROG_NAME PACKAGE
 #define PROG_VERSION PACKAGE_VERSION
 
-#ifdef __PPC__
+#ifdef __x86_64__
+
+#define PROG_CPU "x86_64"
+#ifdef __SSE2__
+#include <emmintrin.h>
+#endif
+#ifdef __SSSE3__
+#include <tmmintrin.h>
+#endif
+
+#elif __PPC__
 
 #ifdef __LITTLE_ENDIAN__
 #define PROG_CPU "ppc64le"
@@ -102,17 +115,14 @@
 #error Big endian ppc64 CPUs not supported
 #endif
 
+#elif __aarch64__
+
+#define PROG_CPU "aarch64"
+#include <arm_neon.h>
+
 #else
 
-#define PROG_CPU "x86_64"
-
-#ifdef __SSE2__
-#include <emmintrin.h>
-#endif
-
-#ifdef __SSSE3__
-#include <tmmintrin.h>
-#endif
+#error Unknown architecture (not ppc64le, aarch64 or x86_64)
 
 #endif
 
@@ -122,6 +132,9 @@
 #define PROG_OS "win"
 #include <windows.h>
 #include <psapi.h>
+#define bswap_16(x) _byteswap_ushort(x)
+#define bswap_32(x) _byteswap_ulong(x)
+#define bswap_64(x) _byteswap_uint64(x)
 
 #else
 
@@ -129,6 +142,10 @@
 
 #define PROG_OS "macos"
 #include <sys/sysctl.h>
+#include <libkern/OSByteOrder.h>
+#define bswap_16(x) OSSwapInt16(x)
+#define bswap_32(x) OSSwapInt32(x)
+#define bswap_64(x) OSSwapInt64(x)
 
 #else
 
@@ -139,6 +156,7 @@
 #endif
 
 #include <sys/sysinfo.h>
+#include <byteswap.h>
 
 #endif
 
@@ -208,6 +226,8 @@
 #include "udb.h"
 #include "kmerhash.h"
 #include "sintax.h"
+#include "fastqjoin.h"
+#include "sffconvert.h"
 
 /* options */
 
@@ -226,6 +246,7 @@ extern bool opt_relabel_keep;
 extern bool opt_relabel_md5;
 extern bool opt_relabel_sha1;
 extern bool opt_samheader;
+extern bool opt_sff_clip;
 extern bool opt_sizeorder;
 extern bool opt_xsize;
 extern char * opt_allpairs_global;
@@ -257,6 +278,7 @@ extern char * opt_fastq_convert;
 extern char * opt_fastq_eestats2;
 extern char * opt_fastq_eestats;
 extern char * opt_fastq_filter;
+extern char * opt_fastq_join;
 extern char * opt_fastq_mergepairs;
 extern char * opt_fastq_stats;
 extern char * opt_fastqout;
@@ -267,6 +289,8 @@ extern char * opt_fastx_filter;
 extern char * opt_fastx_mask;
 extern char * opt_fastx_revcomp;
 extern char * opt_fastx_subsample;
+extern char * opt_join_padgap;
+extern char * opt_join_padgapq;
 extern char * opt_label_suffix;
 extern char * opt_log;
 extern char * opt_makeudb_usearch;
@@ -285,6 +309,7 @@ extern char * opt_rereplicate;
 extern char * opt_reverse;
 extern char * opt_samout;
 extern char * opt_search_exact;
+extern char * opt_sff_convert;
 extern char * opt_shuffle;
 extern char * opt_sintax;
 extern char * opt_sortbylength;
